################################################################
## R wrappers for the functions in random.h
################################################################
##dyn.load("../myrand.so")

## sample from a wishart distribution with E(W) = nu S
my_rwishart = function(n,nu,S)
  {
    d = nrow(S)
    cout <- .C("R_rwishart",
               n = as.integer(n),
               nu = as.integer(nu),
               S = as.double(t(S)),
               dim = as.integer(d),
               x = double(n*d*d))
    aperm(array(cout$x,c(d,d,n)),c(3,1,2))
  }

## sample from an inverse wishart distribution with E(W) = S/(nu-2)
my_rinvwishart = function(n,nu,S)
  {
    d = nrow(S)
    cout <- .C("R_rinvwishart",
               n = as.integer(n),
               nu = as.integer(nu),
               S = as.double(t(S)),
               dim = as.integer(d),
               x = double(n*d*d))
    aperm(array(cout$x,c(d,d,n)),c(3,1,2))
  }

## sample from a multivariate normal mixture with weights p. if p is length J, then
## mu[j,] is the mean of the jth mixture component, and cov[j,,] is the covariance of the jth component
my_rmvnmix = function(n,p,mu,cov)
  {
    dim = ncol(mu)
    ncomp = length(p)
    x = matrix(0,n,dim)
    for(i in 1:n)
      {
        comp = sample(1:ncomp,1,prob=p)
        x[i,] = my_rmvnorm(1,mu[comp,],cov[comp,,])
      }
    x
  }

my_rmvnorm = function(n, mu, cov)
  {
    cout = .C("R_rmvnorm",
      n = as.integer(n),
      mu = as.double(mu),
      cov = as.double(t(cov)),
      dim = as.integer(length(mu)),
      x = double(n*length(mu)))
    matrix(cout$x, nrow=n, ncol=length(mu), byrow=TRUE)
  }

my_rmvtmix = function(n,p,mu,cov,nu)
  {
    dim = ncol(mu)
    ncomp = length(p)
    x = matrix(0,n,dim)
    for(i in 1:n)
      {
        comp = sample(1:ncomp,1,prob=p)
        x[i,] = my_rmvt(1,mu[comp,],cov[comp,,],nu[comp])
      }
    x
  }

my_rmvt = function(n,mu,cov,nu)
  {
    cout = .C("R_rmvt",
      n = as.integer(n),
      mu = as.double(mu),
      cov = as.double(t(cov)),
      nu = as.double(nu),
      dim = as.integer(length(mu)),
      x = double(n*length(mu)))
    matrix(cout$x,nrow=n,ncol=length(mu),byrow=TRUE)
  }

my_rinvgamma = function(n,a,b)
{
  cout = .C("R_rinvgamma",
    n = as.integer(n),
    a = as.double(a),
    b = as.double(b),
    x = double(n))

  cout$x
}

my_rmultinomstar = function(n,counts,p,badZ=matrix(c(3,0,0,0,0,0,3,0,0,0,1,2,0,0,0,2,1,0,0,0),nrow=4,ncol=5,byrow=TRUE))
  {
    cout = .C("R_rmultinomstar",
      n = as.integer(n),
      counts = as.integer(counts),
      p = as.double(p),
      ncats = as.integer(length(p)),
      badx = as.integer(t(badZ)),
      nbad = as.integer(length(badZ[,1])),
      x = integer(n*length(p)))
    matrix(cout$x,nrow=n,ncol=length(p),byrow=TRUE)
  }


my_rdirichlet = function(n, p)
  {
    cout = .C("R_rdirichlet",
      n = as.integer(n),
      p = as.double(p),
      dim = as.integer(length(p)),
      x = double(n*length(p)))

    matrix(cout$x, nrow=n,ncol=length(p),byrow=TRUE)
  }

my_dmultinom = function(x, p, logspace=FALSE)
  {
    cout = .C("R_dmultinom",
      x = as.integer(x),
      p = as.double(p),
      ncats = as.integer(length(p)),
      logspace = as.integer(logspace),
      d = double(1))
    cout$d
  }

my_dmultinomstar = function(x,p,badZ=matrix(c(3,0,0,0,0,0,3,0,0,0,1,2,0,0,0,2,1,0,0,0),nrow=4,ncol=5,byrow=TRUE),logspace=FALSE)
  {
    cout = .C("R_dmultinomstar",
      x = as.integer(x),
      p = as.double(p),
      ncats = as.integer(length(p)),
      badx=as.integer(t(badZ)),
      nbad=as.integer(length(badZ[,1])),
      logspace=as.integer(logspace),
      d=double(1))
    cout$d
  }


my_dmvnorm = function(x, mu, cov,logspace=FALSE)
  {
    cout = .C("R_dmvnorm",
      x = as.double(x),
      mu = as.double(mu),
      cov = as.double(t(cov)),
      dim = as.integer(length(mu)),
      logspace = as.integer(logspace),
      d = double(1))
    cout$d
  }

my_dmvnmix = function(x,p,mu,cov,logspace=FALSE)
  {
    d = rep(0,length(p))
    for(j in 1:length(p))
      {
        d[j] = log(p[j])+my_dmvnorm(x,mu[j,],cov[j,,],TRUE)
      }

    if(logspace)
      dout = log(sum(exp(d)))
    else
      dout = sum(exp(d))
    dout
  }

my_dmvnmixgrid = function(xes,p,mu,cov)
  {
    dim = ncol(xes)
    n = nrow(xes)
    T = length(p)
    
    cout = .C("R_dmvnmixgrid",
      x = as.double(t(xes)),
      n = as.integer(n),
      dim = as.integer(dim),
      p = as.double(p),
      mu = as.double(t(mu)),
      cov = as.double(aperm(cov,3:1)),
      T = as.integer(T),
      dens = double(n))
    cout$dens
  }
    

my_dmvt = function(x, mu, cov, nu, logspace=FALSE)
  {
    cout = .C("R_dmvt",
      x = as.double(x),
      mu = as.double(mu),
      cov = as.double(t(cov)),
      nu = as.double(nu),
      dim = as.integer(length(mu)),
      logspace = as.integer(logspace),
      d = double(1))
    cout$d
  }

my_dmvtmix = function(x,p,mu,cov,nu,logspace=FALSE)
  {
    d = rep(0,length(p))
    for(j in 1:length(p))
      {
        d[j] = log(p[j])+my_dmvt(x,mu[j,],cov[j,,],nu[j],TRUE)
      }

    if(logspace)
      dout = log(sum(exp(d)))
    else
      dout = sum(exp(d))
    dout
  }
my_dmvtmixgrid = function(xes,q,mu,cov,nu)
  {
    dim= ncol(xes)
    n = nrow(xes)
    J = length(q)
    cout = .C("R_dmvtmixgrid",
      x = as.double(t(xes)),
      n = as.integer(n),
      dim = as.integer(dim),
      p = as.double(q),
      mu = as.double(t(mu)),
      cov = as.double(aperm(cov,3:1)),
      nu = as.double(nu),
      T = as.integer(J),
      dens = double(n))
    cout$dens
  }
my_dinvgamma = function(x,a,b,logspace=FALSE)
  {
    cout = .C("R_dinvgamma",
      x = as.double(x),
      a = as.double(a),
      b = as.double(b),
      logspace = as.integer(logspace),
      d = double(1))
    cout$d
  }

my_ddirichlet = function(x,p,logspace=FALSE)
  {
    cout = .C("R_ddirichlet",
      x = as.double(x),
      p = as.double(p),
      dim = as.integer(length(x)),
      logspace = as.integer(logspace),
      d = double(1))
    cout$d
  }

## below here used for testing, but not needed since R already has them

## my_dunif = function(x,a,b,logspace=FALSE)
##   {
##     cout = .C("R_dunif",
##       x = as.double(x),
##       a = as.double(a),
##       b = as.double(b),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dgamma = function(x,a,b,logspace=FALSE)
##   {
##     cout = .C("R_dgamma",
##       x = as.double(x),
##       a = as.double(a),
##       b = as.double(b),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dbeta = function(x,a,b,logspace=FALSE)
##   {
##     cout = .C("R_dbeta",
##       x = as.double(x),
##       a = as.double(a),
##       b = as.double(b),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dbinom = function(x, counts, p, logspace=FALSE)
##   {
##     cout = .C("R_dbinom",
##       x = as.integer(x),
##       counts = as.integer(counts),
##       p = as.double(p),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dexp = function(x,la,logspace=FALSE)
##   {
##     cout = .C("R_dexp",
##       x = as.double(x),
##       la = as.double(la),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dgeom = function(x,p,logspace=FALSE)
##   {
##     cout = .C("R_dgeom",
##       x = as.integer(x),
##       p = as.double(p),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dpois = function(x,la,logspace=FALSE)
##   {
##     cout = .C("R_dpois",
##       x = as.integer(x),
##       la = as.double(la),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_dnorm = function(x,mu,var,logspace=FALSE)
##   {
##     cout = .C("R_dnorm",
##       x = as.double(x),
##       mu = as.double(mu),
##       var = as.double(var),
##       logspace = as.integer(logspace),
##       d = double(1))
##     cout$d
##   }

## my_runif = function(n,a,b)
## {
##   cout = .C("R_runif",
##     n = as.integer(n),
##     a = as.double(a),
##     b = as.double(b),
##     x = double(n))
##   cout$x
## }

## my_rgamma = function(n,a,b)
## {
##   cout = .C("R_rgamma",
##     n = as.integer(n),
##     a = as.double(a),
##     b = as.double(b),
##     x = double(n))

##   cout$x
## }

## my_rbeta = function(n,a,b)
## {
##   cout = .C("R_rbeta",
##     n = as.integer(n),
##     a = as.double(a),
##     b = as.double(b),
##     x = double(n))

##   cout$x
## }

## my_rbinom = function(n,counts, p)
##   {
##     cout = .C("R_rbinom",
##       n = as.integer(n),
##       counts = as.integer(counts),
##       p = as.double(p),
##       x = integer(n))
##     cout$x
##   }

## my_rmultinom = function(n, counts, p)
##   {
##     cout = .C("R_rmultinom",
##       n = as.integer(n),
##       counts = as.integer(counts),
##       p = as.double(p),
##       ncats = as.integer(length(p)),
##       x = integer(n*length(p)))

##     matrix(cout$x, nrow=n, ncol=length(p),byrow=TRUE)
##   }

## my_rexp = function(n,la)
##   {
##     cout = .C("R_rexp",
##       n = as.integer(n),
##       la = as.double(la),
##       x = double(n))
##     cout$x
##   }

## my_rgeom = function(n, p)
##   {
##     cout = .C("R_rgeom",
##       n = as.integer(n),
##       p = as.double(p),
##       x = integer(n))
##     cout$x
##   }

## my_rpois = function(n, la)
##   {
##     cout = .C("R_rpois",
##       n = as.integer(n),
##       la = as.double(la),
##       x = integer(n))
##     cout$x
##   }

## my_rnorm = function(n, mu, sd)
##   {
##     cout = .C("R_rnorm",
##       n = as.integer(n),
##       mu = as.double(mu),
##       sd = as.double(sd),
##       x = double(n))
##     cout$x
##   }

## dmvnormtest = function(x,mu,cov)
##   {
##     d = length(x)

##     c = (2*pi)^(-d/2)/sqrt(det(cov))

##     k = exp(-.5 * t(x-mu)%*%solve(cov)%*%(x-mu))

##     c*k
##   }

## my_chol = function(cov)
##   {
##     d = length(cov[1,])
##     cout = .C("R_chol",
##               cov = as.double(t(cov)),
##               dim = as.integer(d))
##     matrix(cout$cov, d,d,byrow=TRUE)
##   }
