
#ifdef DEBUG
#include <assert.h>
#endif

#include <math.h>
#include "linalg.h"
#include "matrix.h"
#include <stdlib.h>

char uplo = 'U';


/*******************************
 Matrix Inversion Stuff
*******************************/
void my_dgetri(int n, double** A, int lda, int* ipiv, double* work, int lwork, int* info)
{
  dgetri(&n, *A, &lda, ipiv, work, &lwork, info);
}

/* void R_dgetri(int* n, double* A, int* lda, int* ipiv, double* work, int* lwork, int* info) */
/* { */
/*   my_dgetri(*n,&A,*lda,ipiv,work,*lwork,info); */
/* } */

void my_dgetrf(int m, int n, double** A, int lda, int* ipiv, int* info)
{
  dgetrf(&m, &n, *A, &lda, ipiv, info);
}

void my_invert_matrix(double** A, int dim)
{
  int *ipiv, info;
  double *work;
  work = new_vector(dim);
  ipiv = new_ivector(dim);
  my_dgetrf(dim,dim,A,dim,ipiv,&info);
  my_dgetri(dim,A,dim,ipiv,work,dim,&info);
  free(ipiv);
  free(work);
}

void R_invertmatrix(double* A, int* dim)
{
  my_invert_matrix(&A,*dim);
}

/********************************
 Cholesky stuff
********************************/

/* my_dpotrf: cholesky decomposition
 *
 * Assumes cov a rowmajor nxn symmetric positive semi-definite matrix
 * Writes the cholesky factor in the lower triangular, with pivots on 
 * the diagonal.
 */
int my_dpotrf(cov, n)
double **cov;
int n;
{
  int info;
  
  dpotrf(&uplo,&n,*cov,&n,&info); 
#ifdef DEBUG
  assert(info == 0);
#endif
  
  return info;
}

/*
  calculates the cholesky decomposition of a covariance matrix
  this function will allow singular matrices, for which a row and column 
  have all zero entries.  This allows us to represent mixtures of point masses
  and multivariate distributions using a single multivariate normal representation 
*/
void my_chol(double** cov, int dim)
{
  my_dpotrf(cov, dim);
      
}

/*
 * Allows calls to the cholesky factorization routine from R
 * Returns the cholesky factor in the lower triangular
 */
void R_chol(double* cov, int* dim)
{
  double** mycov;
  mycov = new_2ddptrs(*dim,*dim,cov);
  my_chol(mycov, *dim);
  rm_2ddptrs(mycov);
}

/*
 * calculates the determinant of a matrix
 * assumes cov is row major, with the cholesky factor in the lower triangular
 */
double my_det(double** cov, int dim, int logspace){
  int i;
  double d;
  
  d = 0.;
  
  for(i=0;i<dim;i++)
    d+=log(cov[i][i]);

  d = 2.*d;
  if(!logspace)
    d = exp(d);

  return d;
}


/**************************************
  Matrix Multiplication
*************************************/

/*
 * my_dgemm:  C:= alpha*op(A)*op(B) + beta*C
 */
void my_dgemm(TA,TB,m,n,k,alpha,A,lda,B,ldb,beta,C,ldc)
     const enum CBLAS_TRANSPOSE TA,TB;
     int m,n,k,lda,ldb,ldc;
     double alpha,beta;
     double **A,**B,**C;
{
  char ta,tb;
  if(TA==CblasTrans) ta='T';else ta='N';
  if(TB==CblasTrans) tb='T';else tb='N';
  dgemm(&ta,&tb,&m,&n,&k,&alpha,*A,&lda,*B,&ldb,&beta,*C,&ldc);
}

void R_mymatmult(double *A, int* nrowa, int* ncola, int* transa,
		 double *B, int* nrowb, int* ncolb, int* transb,
		 double* C)
{
  my_matrix_multiply(&A,*nrowa,*ncola,*transa,
		     &B,*nrowb,*ncolb,*transb,
		     &C);
}

/*  assumes A and B are row major */
void my_matrix_multiply(double** A, int nrowa, int ncola, int transa, 
			double** B, int nrowb, int ncolb, int transb,
			double** C)
{
  double foo1,foo2;
  foo1=1; foo2=0;
  char ta,tb;
  if(transa)
    ta = 'T';
  else
    ta = 'N';
  if(transb)
    tb = 'T';
  else
    tb = 'N';
  
  if(!transa && !transb)
    dgemm(&tb,&ta, &ncolb,&nrowa,&ncola,&foo1,*B,&ncolb,*A,&ncola,&foo2,*C,&ncolb);
  if(transa && transb)
    dgemm(&tb,&ta, &nrowb,&ncola,&nrowa,&foo1,*B,&ncolb,*A,&ncola,&foo2,*C,&nrowb);
  if(transa && !transb)
    dgemm(&tb,&ta,&ncolb,&ncola,&nrowa,&foo1,*B,&ncolb,*A,&ncola,&foo2,*C,&ncolb);
  if(!transa && transb)
    dgemm(&tb,&ta,&nrowb,&nrowa,&ncola,&foo1,*B,&ncolb,*A,&ncola,&foo2,*C,&nrowb);
     
}

void R_dgemm(char** ta, char** tb, int* m, int* n, int* k, double* alpha, double* A, int* lda, double* B, int* ldb, double* beta, double* C, int* ldc)
{
  dgemm(*ta, *tb, m, n, k, alpha, A, lda, B, ldb, beta, C, ldc);
}


/*******************************
  Other matrix/vector stuff
******************************/

/*
 * my_dgemv: y:= alpha*A*x + beta*y or y:= alpha*A'*x + beta*y
 *
 * Assumes all matrices are row major
 *
 * A is m x n matrix
 * if TA = CblasTrans, y:= alpha * A * x + beta*y
 * if TA = CblasNoTrans, y:= alpha* x * A + beta * y
 */

void my_dgemv(TA, m, n, alpha, A, lda, X, ldx, beta, Y, ldy)
const enum CBLAS_TRANSPOSE TA;
int m, n, lda, ldx, ldy;
double alpha, beta;
double **A;
double *X, *Y;
{
  char ta;
  if(TA == CblasTrans) ta = 'T'; else ta = 'N';
  dgemv(&ta,&n,&m,&alpha,*A,&lda,X,&ldx,&beta,Y,&ldy);
}


/*
 * my_ddot:computes the dot product of X and Y
 *
 * analog of ddot in cblas nad blas
 */

double my_ddot(n, X, ldx, Y, ldy)
int n, ldx, ldy;
double *X, *Y;
{
  double result;
  result = ddot(&n,X,&ldx,Y,&ldy);
  return result;
}

/*
 * my_dtrsv: solves the system A*x = y or A'*x = y for x
 *
 * The value of x is written over the value of Y
 * Assumes A is an upper triangular rowmajor matrix, use TA to 
 * transpose a lower triangular to an upper triangular  
 */

void my_dtrsv(TA, n, A, lda, Y, ldy)
const enum CBLAS_TRANSPOSE TA;
int n, lda, ldy;
double **A;
double *Y;
{
  char ta;
  char diag = 'N';
  if(TA == CblasTrans) ta = 'T'; else ta = 'N';
  dtrsv(&uplo, &ta, &diag, &n, *A, &lda, Y, &ldy);
}

/*
 * my_daxpy: y:=alpha*x+y
 *
 * analog of daxpy in cblas nad blas
 */
void my_daxpy(n,alpha,X,ldx,Y,ldy)
int n, ldx, ldy;
double alpha;
double *X, *Y;
{
  daxpy(&n,&alpha,X,&ldx,Y,&ldy);
}


/*
 * my_dsymm: performs C:= alpha*A*B + beta*C or
 *                    C:= alpha*B*A + beta*C
 *
 * Assumes all matrices are row major, A is symmetric
 *
 * if SIDE = CblasRight, C:=alpha*A*B + beta*C
 * if SIDE = CblasLeft, C:=alpha*B*A + beta*C
 * Output matrix C is m x n 
 * lda is how many entries are between rows of A 
 * ldb is how many entries are between rows of B 
 * ldc is how many entries are between rows of C 
 *
 * if A is m x m and B is m x n, and we want A*B
 * my_dsymm(CblasRight, m, n, 1, A, m, B, n, 0, C, n)
 *
 * if A is n x n and B is m x n and we want B*A
 * my_dsymm(CblasLeft, m, n, 1, A, n, B, n, C, n)
 */

/* void my_dsymm(SIDE, m, n, alpha, A, lda, B, ldb, beta, C, ldc) */
/* const enum CBLAS_SIDE SIDE; */
/* int m, n, lda, ldb, ldc; */
/* double alpha, beta; */
/* double **A, **B, **C; */
/* { */
/*   char side; */
/*   if(SIDE == CblasRight) side = 'R'; else side = 'L'; */
/*   dsymm(&side,&uplo,&n,&m,&alpha,*A,&lda,*B,&ldb,&beta,*C,&ldc); */
/* } */

